@@ -182,6 +182,7 @@ finish_socket_setup (SoupSocketPrivate *priv)
 		priv->ostream = g_object_ref (g_io_stream_get_output_stream (priv->iostream));
 
 	g_socket_set_timeout (priv->gsock, priv->timeout);
+	g_socket_set_option (priv->gsock, IPPROTO_TCP, TCP_NODELAY, TRUE, NULL);
 }
 
 static void
@@ -666,38 +667,36 @@ re_emit_socket_client_event (GSocketClient       *client,
 		       event, connection);
 }
 
-static guint
-socket_connected (SoupSocket *sock, GSocketConnection *conn, GError *error)
+static gboolean
+socket_connect_finish (SoupSocket *sock, GSocketConnection *conn)
 {
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
-	if (priv->connect_cancel) {
-		GCancellable *cancellable = priv->connect_cancel;
-		gboolean cancelled = g_cancellable_is_cancelled (cancellable);
+	g_clear_object (&priv->connect_cancel);
 
-		g_object_unref (priv->connect_cancel);
-		priv->connect_cancel = NULL;
-		if (cancelled) {
-			g_clear_error (&error);
-			return SOUP_STATUS_CANCELLED;
-		}
-	}
+	if (conn) {
+		priv->conn = (GIOStream *)conn;
+		priv->gsock = g_object_ref (g_socket_connection_get_socket (conn));
+		finish_socket_setup (priv);
+		return TRUE;
+	} else
+		return FALSE;
+}
 
-	if (error) {
-		if (error->domain == G_RESOLVER_ERROR) {
-			g_error_free (error);
-			return SOUP_STATUS_CANT_RESOLVE;
-		} else {
-			g_error_free (error);
-			return SOUP_STATUS_CANT_CONNECT;
-		}
-	}
+static guint
+socket_legacy_error (SoupSocket *sock, GError *error)
+{
+	guint status;
 
-	priv->conn = (GIOStream *)conn;
-	priv->gsock = g_object_ref (g_socket_connection_get_socket (conn));
-	finish_socket_setup (priv);
+	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+		status = SOUP_STATUS_CANCELLED;
+	else if (error->domain == G_RESOLVER_ERROR)
+		status = SOUP_STATUS_CANT_RESOLVE;
+	else
+		status = SOUP_STATUS_CANT_CONNECT;
 
-	return SOUP_STATUS_OK;
+	g_error_free (error);
+	return status;
 }
 
 static GSocketClient *
@@ -722,6 +721,58 @@ new_socket_client (SoupSocket *sock)
 	return client;
 }
 
+static void
+async_connected (GObject *client, GAsyncResult *result, gpointer data)
+{
+	GTask *task = data;
+	SoupSocket *sock = g_task_get_source_object (task);
+	GSocketConnection *conn;
+	GError *error = NULL;
+
+	conn = g_socket_client_connect_finish (G_SOCKET_CLIENT (client),
+					       result, &error);
+	if (socket_connect_finish (sock, conn))
+		g_task_return_boolean (task, TRUE);
+	else
+		g_task_return_error (task, error);
+	g_object_unref (task);
+}
+
+gboolean
+soup_socket_connect_finish_internal (SoupSocket    *sock,
+				     GAsyncResult  *result,
+				     GError       **error)
+{
+	return g_task_propagate_boolean (G_TASK (result), error);
+}
+
+void
+soup_socket_connect_async_internal (SoupSocket          *sock,
+				    GCancellable        *cancellable,
+				    GAsyncReadyCallback  callback,
+				    gpointer             user_data)
+{
+	SoupSocketPrivate *priv;
+	GSocketClient *client;
+	GTask *task;
+
+	g_return_if_fail (SOUP_IS_SOCKET (sock));
+	priv = SOUP_SOCKET_GET_PRIVATE (sock);
+	g_return_if_fail (!priv->is_server);
+	g_return_if_fail (priv->gsock == NULL);
+	g_return_if_fail (priv->remote_addr != NULL);
+
+	priv->connect_cancel = cancellable ? g_object_ref (cancellable) : g_cancellable_new ();
+	task = g_task_new (sock, priv->connect_cancel, callback, user_data);
+
+	client = new_socket_client (sock);
+	g_socket_client_connect_async (client,
+				       G_SOCKET_CONNECTABLE (priv->remote_addr),
+				       priv->connect_cancel,
+				       async_connected, task);
+	g_object_unref (client);
+}
+
 /**
  * SoupSocketCallback:
  * @sock: the #SoupSocket
@@ -738,22 +789,25 @@ typedef struct {
 } SoupSocketAsyncConnectData;
 
 static void
-async_connected (GObject *client, GAsyncResult *result, gpointer data)
+legacy_connect_async_cb (GObject       *object,
+			 GAsyncResult  *result,
+			 gpointer       user_data)
 {
-	SoupSocketAsyncConnectData *sacd = data;
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sacd->sock);
+	SoupSocket *sock = SOUP_SOCKET (object);
+	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
+	SoupSocketAsyncConnectData *sacd = user_data;
 	GError *error = NULL;
-	GSocketConnection *conn;
 	guint status;
 
 	if (priv->async_context && !priv->use_thread_context)
 		g_main_context_pop_thread_default (priv->async_context);
 
-	conn = g_socket_client_connect_finish (G_SOCKET_CLIENT (client),
-					       result, &error);
-	status = socket_connected (sacd->sock, conn, error);
+	if (soup_socket_connect_finish_internal (sock, result, &error))
+		status = SOUP_STATUS_OK;
+	else
+		status = socket_legacy_error (sock, error);
 
-	sacd->callback (sacd->sock, status, sacd->user_data);
+	sacd->callback (sock, status, sacd->user_data);
 	g_object_unref (sacd->sock);
 	g_slice_free (SoupSocketAsyncConnectData, sacd);
 }
@@ -779,10 +833,11 @@ soup_socket_connect_async (SoupSocket *sock, GCancellable *cancellable,
 {
 	SoupSocketPrivate *priv;
 	SoupSocketAsyncConnectData *sacd;
-	GSocketClient *client;
 
 	g_return_if_fail (SOUP_IS_SOCKET (sock));
 	priv = SOUP_SOCKET_GET_PRIVATE (sock);
+	g_return_if_fail (!priv->is_server);
+	g_return_if_fail (priv->gsock == NULL);
 	g_return_if_fail (priv->remote_addr != NULL);
 
 	sacd = g_slice_new0 (SoupSocketAsyncConnectData);
@@ -790,17 +845,38 @@ soup_socket_connect_async (SoupSocket *sock, GCancellable *cancellable,
 	sacd->callback = callback;
 	sacd->user_data = user_data;
 
-	priv->connect_cancel = cancellable ? g_object_ref (cancellable) : g_cancellable_new ();
-
 	if (priv->async_context && !priv->use_thread_context)
 		g_main_context_push_thread_default (priv->async_context);
 
+	soup_socket_connect_async_internal (sock, cancellable,
+					    legacy_connect_async_cb,
+					    sacd);
+}
+
+gboolean
+soup_socket_connect_sync_internal (SoupSocket    *sock,
+				   GCancellable  *cancellable,
+				   GError       **error)
+{
+	SoupSocketPrivate *priv;
+	GSocketClient *client;
+	GSocketConnection *conn;
+
+	g_return_val_if_fail (SOUP_IS_SOCKET (sock), SOUP_STATUS_MALFORMED);
+	priv = SOUP_SOCKET_GET_PRIVATE (sock);
+	g_return_val_if_fail (!priv->is_server, SOUP_STATUS_MALFORMED);
+	g_return_val_if_fail (priv->gsock == NULL, SOUP_STATUS_MALFORMED);
+	g_return_val_if_fail (priv->remote_addr != NULL, SOUP_STATUS_MALFORMED);
+
+	priv->connect_cancel = cancellable ? g_object_ref (cancellable) : g_cancellable_new ();
+
 	client = new_socket_client (sock);
-	g_socket_client_connect_async (client,
-				       G_SOCKET_CONNECTABLE (priv->remote_addr),
-				       priv->connect_cancel,
-				       async_connected, sacd);
+	conn = g_socket_client_connect (client,
+					G_SOCKET_CONNECTABLE (priv->remote_addr),
+					priv->connect_cancel, error);
 	g_object_unref (client);
+
+	return socket_connect_finish (sock, conn);
 }
 
 /**
@@ -820,8 +896,6 @@ guint
 soup_socket_connect_sync (SoupSocket *sock, GCancellable *cancellable)
 {
 	SoupSocketPrivate *priv;
-	GSocketClient *client;
-	GSocketConnection *conn;
 	GError *error = NULL;
 
 	g_return_val_if_fail (SOUP_IS_SOCKET (sock), SOUP_STATUS_MALFORMED);
@@ -830,19 +904,10 @@ soup_socket_connect_sync (SoupSocket *sock, GCancellable *cancellable)
 	g_return_val_if_fail (priv->gsock == NULL, SOUP_STATUS_MALFORMED);
 	g_return_val_if_fail (priv->remote_addr != NULL, SOUP_STATUS_MALFORMED);
 
-	if (cancellable)
-		g_object_ref (cancellable);
+	if (soup_socket_connect_sync_internal (sock, cancellable, &error))
+		return SOUP_STATUS_OK;
 	else
-		cancellable = g_cancellable_new ();
-	priv->connect_cancel = cancellable;
-
-	client = new_socket_client (sock);
-	conn = g_socket_client_connect (client,
-					G_SOCKET_CONNECTABLE (priv->remote_addr),
-					priv->connect_cancel, &error);
-	g_object_unref (client);
-
-	return socket_connected (sock, conn, error);
+		return socket_legacy_error (sock, error);
 }
 
 /**
@@ -1035,37 +1100,11 @@ soup_socket_accept_certificate (GTlsConnection *conn, GTlsCertificate *cert,
 	return TRUE;
 }
 
-/**
- * soup_socket_start_ssl:
- * @sock: the socket
- * @cancellable: a #GCancellable
- *
- * Starts using SSL on @socket.
- *
- * Return value: success or failure
- **/
-gboolean
-soup_socket_start_ssl (SoupSocket *sock, GCancellable *cancellable)
-{
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
-
-	return soup_socket_start_proxy_ssl (sock, soup_address_get_name (priv->remote_addr), cancellable);
-}
-	
-/**
- * soup_socket_start_proxy_ssl:
- * @sock: the socket
- * @ssl_host: hostname of the SSL server
- * @cancellable: a #GCancellable
- *
- * Starts using SSL on @socket, expecting to find a host named
- * @ssl_host.
- *
- * Return value: success or failure
- **/
-gboolean
-soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
-			     GCancellable *cancellable)
+static gboolean
+soup_socket_setup_ssl (SoupSocket    *sock,
+		       const char    *ssl_host,
+		       GCancellable  *cancellable,
+		       GError       **error)
 {
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 	GTlsBackend *backend = g_tls_backend_get_default ();
@@ -1073,7 +1112,7 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 	if (G_IS_TLS_CONNECTION (priv->conn))
 		return TRUE;
 
-	if (g_cancellable_is_cancelled (cancellable))
+	if (g_cancellable_set_error_if_cancelled (cancellable, error))
 		return FALSE;
 
 	priv->ssl = TRUE;
@@ -1084,7 +1123,7 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 
 		identity = g_network_address_new (ssl_host, 0);
 		conn = g_initable_new (g_tls_backend_get_client_connection_type (backend),
-				       NULL, NULL,
+				       cancellable, error,
 				       "base-io-stream", priv->conn,
 				       "server-identity", identity,
 				       "database", priv->ssl_creds,
@@ -1108,7 +1147,7 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 		GTlsServerConnection *conn;
 
 		conn = g_initable_new (g_tls_backend_get_server_connection_type (backend),
-				       NULL, NULL,
+				       cancellable, error,
 				       "base-io-stream", priv->conn,
 				       "certificate", priv->ssl_creds,
 				       "use-system-certdb", FALSE,
@@ -1133,76 +1172,103 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 
 	return TRUE;
 }
-	
-guint
-soup_socket_handshake_sync (SoupSocket    *sock,
-			    GCancellable  *cancellable)
+
+/**
+ * soup_socket_start_ssl:
+ * @sock: the socket
+ * @cancellable: a #GCancellable
+ *
+ * Starts using SSL on @socket.
+ *
+ * Return value: success or failure
+ **/
+gboolean
+soup_socket_start_ssl (SoupSocket *sock, GCancellable *cancellable)
 {
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
-	GError *error = NULL;
 
-	priv->ssl = TRUE;
-	if (g_tls_connection_handshake (G_TLS_CONNECTION (priv->conn),
-					cancellable, &error))
-		return SOUP_STATUS_OK;
-	else if (!priv->ssl_fallback &&
-		 g_error_matches (error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS)) {
-		g_error_free (error);
-		return SOUP_STATUS_TLS_FAILED;
-	} else {
-		g_error_free (error);
-		return SOUP_STATUS_SSL_FAILED;
-	}
+	return soup_socket_setup_ssl (sock, soup_address_get_name (priv->remote_addr),
+				      cancellable, NULL);
+}
+	
+/**
+ * soup_socket_start_proxy_ssl:
+ * @sock: the socket
+ * @ssl_host: hostname of the SSL server
+ * @cancellable: a #GCancellable
+ *
+ * Starts using SSL on @socket, expecting to find a host named
+ * @ssl_host.
+ *
+ * Return value: success or failure
+ **/
+gboolean
+soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
+			     GCancellable *cancellable)
+{
+	return soup_socket_setup_ssl (sock, ssl_host, cancellable, NULL);
+}
+
+gboolean
+soup_socket_handshake_sync (SoupSocket    *sock,
+			    const char    *ssl_host,
+			    GCancellable  *cancellable,
+			    GError       **error)
+{
+	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
+
+	if (!soup_socket_setup_ssl (sock, ssl_host, cancellable, error))
+		return FALSE;
+
+	return g_tls_connection_handshake (G_TLS_CONNECTION (priv->conn),
+					   cancellable, error);
 }
 
 static void
 handshake_async_ready (GObject *source, GAsyncResult *result, gpointer user_data)
 {
-	SoupSocketAsyncConnectData *data = user_data;
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (data->sock);
+	GTask *task = user_data;
 	GError *error = NULL;
-	guint status;
-
-	if (priv->async_context && !priv->use_thread_context)
-		g_main_context_pop_thread_default (priv->async_context);
 
 	if (g_tls_connection_handshake_finish (G_TLS_CONNECTION (source),
 					       result, &error))
-		status = SOUP_STATUS_OK;
-	else if (!priv->ssl_fallback &&
-		 g_error_matches (error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS))
-		status = SOUP_STATUS_TLS_FAILED;
+		g_task_return_boolean (task, TRUE);
 	else
-		status = SOUP_STATUS_SSL_FAILED;
-	g_clear_error (&error);
-
-	data->callback (data->sock, status, data->user_data);
-	g_object_unref (data->sock);
-	g_slice_free (SoupSocketAsyncConnectData, data);
+		g_task_return_error (task, error);
+	g_object_unref (task);
 }
 
 void
-soup_socket_handshake_async (SoupSocket         *sock,
-			     GCancellable       *cancellable,
-			     SoupSocketCallback  callback,
-			     gpointer            user_data)
+soup_socket_handshake_async (SoupSocket          *sock,
+			     const char          *ssl_host,
+			     GCancellable        *cancellable,
+			     GAsyncReadyCallback  callback,
+			     gpointer             user_data)
 {
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
-	SoupSocketAsyncConnectData *data;
+	GTask *task;
+	GError *error = NULL;
 
-	priv->ssl = TRUE;
+	task = g_task_new (sock, cancellable, callback, user_data);
 
-	data = g_slice_new (SoupSocketAsyncConnectData);
-	data->sock = g_object_ref (sock);
-	data->callback = callback;
-	data->user_data = user_data;
+	if (!soup_socket_setup_ssl (sock, ssl_host, cancellable, &error)) {
+		g_task_return_error (task, error);
+		g_object_unref (task);
+		return;
+	}
 
-	if (priv->async_context && !priv->use_thread_context)
-		g_main_context_push_thread_default (priv->async_context);
 	g_tls_connection_handshake_async (G_TLS_CONNECTION (priv->conn),
 					  G_PRIORITY_DEFAULT,
 					  cancellable, handshake_async_ready,
-					  data);
+					  task);
+}
+
+gboolean
+soup_socket_handshake_finish (SoupSocket    *sock,
+			      GAsyncResult  *result,
+			      GError       **error)
+{
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 /**
