@@ -3,33 +3,42 @@
 #include "test-utils.h"
 
 static void
-do_ssl_test_for_session (SoupSession *session, SoupURI *uri)
+do_ssl_test_for_session (SoupSession *session, char *uri)
 {
 	SoupMessage *msg;
-	GTlsCertificate *cert = NULL;
+	GTlsCertificate *cert;
 	GTlsCertificateFlags flags;
-	gboolean is_https;
 
-	msg = soup_message_new_from_uri ("GET", uri);
+	msg = soup_message_new ("GET", uri);
 	soup_session_send_message (session, msg);
-	soup_test_assert_message_status (msg, SOUP_STATUS_SSL_FAILED);
+	if (msg->status_code != SOUP_STATUS_SSL_FAILED) {
+		debug_printf (1, "    Unexpected status: %d %s\n",
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
 
-	is_https = soup_message_get_https_status (msg, &cert, &flags);
-	soup_test_assert (!is_https, "get_http_status() returned TRUE? (flags %x)", flags);
-
-	g_assert_null (cert);
-	g_assert_false (soup_message_get_flags (msg) & SOUP_MESSAGE_CERTIFICATE_TRUSTED);
+	if (soup_message_get_https_status (msg, &cert, &flags)) {
+		debug_printf (1, "    get_http_status() returned TRUE? (flags %x)\n", flags);
+		errors++;
+		if (cert) {
+			debug_printf (1, "    Got GTlsCertificate?\n");
+			errors++;
+		}
+	}
+	if (soup_message_get_flags (msg) & SOUP_MESSAGE_CERTIFICATE_TRUSTED) {
+		debug_printf (1, "    CERTIFICATE_TRUSTED set?\n");
+		errors++;
+	}
 
 	g_object_unref (msg);
 }
 
 static void
-do_ssl_tests (gconstpointer data)
+do_ssl_tests (char *uri)
 {
-	SoupURI *uri = (SoupURI *)data;
 	SoupSession *session;
 
-	g_test_bug ("700518");
+	debug_printf (1, "\nSoupSession without SSL support\n");
 
 	debug_printf (1, "  plain\n");
 	session = soup_test_session_new (SOUP_TYPE_SESSION, NULL);
@@ -54,27 +63,30 @@ do_session_property_tests (void)
 	GTlsDatabase *tlsdb;
 	char *ca_file;
 	SoupSession *session;
-	GParamSpec *pspec;
 
-	g_test_bug ("700518");
+	debug_printf (1, "session properties\n");
 
-	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 	session = soup_session_async_new ();
-	G_GNUC_END_IGNORE_DEPRECATIONS;
-
-	/* Temporarily undeprecate SOUP_SESSION_SSL_CA_FILE to avoid warnings. */
-	pspec = g_object_class_find_property (g_type_class_peek (SOUP_TYPE_SESSION),
-					      SOUP_SESSION_SSL_CA_FILE);
-	pspec->flags &= ~G_PARAM_DEPRECATED;
 
 	g_object_get (G_OBJECT (session),
 		      "ssl-use-system-ca-file", &use_system,
 		      "tls-database", &tlsdb,
 		      "ssl-ca-file", &ca_file,
 		      NULL);
-	soup_test_assert (!use_system, "ssl-use-system-ca-file defaults to TRUE");
-	soup_test_assert (tlsdb == NULL, "tls-database set by default");
-	soup_test_assert (ca_file == NULL, "ca-file set by default");
+	if (use_system) {
+		debug_printf (1, "  ssl-use-system-ca-file defaults to TRUE?\n");
+		errors++;
+	}
+	if (tlsdb) {
+		debug_printf (1, "  tls-database set by default?\n");
+		errors++;
+		g_object_unref (tlsdb);
+	}
+	if (ca_file) {
+		debug_printf (1, "  ca-file set by default?\n");
+		errors++;
+		g_free (ca_file);
+	}
 
 	g_object_set (G_OBJECT (session),
 		      "ssl-use-system-ca-file", TRUE,
@@ -82,20 +94,34 @@ do_session_property_tests (void)
 	g_object_get (G_OBJECT (session),
 		      "ssl-ca-file", &ca_file,
 		      NULL);
-	soup_test_assert (ca_file == NULL, "setting ssl-use-system-ca-file set ssl-ca-file");
+	if (ca_file) {
+		debug_printf (1, "  setting ssl-use-system-ca-file set ssl-ca-file\n");
+		errors++;
+		g_free (ca_file);
+	}
 
 	g_object_set (G_OBJECT (session),
-		      "ssl-ca-file",
-		      g_test_get_filename (G_TEST_DIST, "test-cert.pem", NULL),
+		      "ssl-ca-file", SRCDIR "/test-cert.pem",
 		      NULL);
 	g_object_get (G_OBJECT (session),
 		      "ssl-use-system-ca-file", &use_system,
 		      "tls-database", &tlsdb,
 		      "ssl-ca-file", &ca_file,
 		      NULL);
-	soup_test_assert (ca_file == NULL, "setting ssl-ca-file did not fail");
-	soup_test_assert (!use_system, "setting ssl-ca-file set ssl-use-system-ca-file");
-	soup_test_assert (tlsdb == NULL, "setting ssl-ca-file set tls-database");
+	if (ca_file) {
+		debug_printf (1, "  setting ssl-ca-file did not fail\n");
+		errors++;
+		g_free (ca_file);
+	}
+	if (use_system) {
+		debug_printf (1, "  setting ssl-ca-file set ssl-use-system-ca-file\n");
+		errors++;
+	}
+	if (tlsdb) {
+		debug_printf (1, "  setting ssl-ca-file set tls-database\n");
+		errors++;
+		g_object_unref (tlsdb);
+	}
 
 	g_object_set (G_OBJECT (session),
 		      "tls-database", NULL,
@@ -105,19 +131,27 @@ do_session_property_tests (void)
 		      "tls-database", &tlsdb,
 		      "ssl-ca-file", &ca_file,
 		      NULL);
-	soup_test_assert (tlsdb == NULL, "setting tls-database NULL failed");
-	soup_test_assert (!use_system, "setting tls-database NULL set ssl-use-system-ca-file");
-	soup_test_assert (ca_file == NULL, "setting tls-database NULL set ssl-ca-file");
+	if (tlsdb) {
+		debug_printf (1, "  setting tls-database NULL failed\n");
+		errors++;
+		g_object_unref (tlsdb);
+	}
+	if (use_system) {
+		debug_printf (1, "  setting tls-database NULL set ssl-use-system-ca-file\n");
+		errors++;
+	}
+	if (ca_file) {
+		debug_printf (1, "  setting tls-database NULL set ssl-ca-file\n");
+		errors++;
+		g_free (ca_file);
+	}
 
 	soup_test_session_abort_unref (session);
-
-	/* Re-deprecate SOUP_SESSION_SSL_CA_FILE */
-	pspec->flags |= G_PARAM_DEPRECATED;
 }
 
 static void
 server_handler (SoupServer        *server,
-		SoupMessage       *msg,
+		SoupMessage       *msg, 
 		const char        *path,
 		GHashTable        *query,
 		SoupClientContext *client,
@@ -133,9 +167,7 @@ int
 main (int argc, char **argv)
 {
 	SoupServer *server;
-	SoupURI *uri;
-	guint port;
-	int ret;
+	char *uri;
 
 	/* Force this test to use the dummy TLS backend */
 	g_setenv ("GIO_USE_TLS", "dummy", TRUE);
@@ -148,19 +180,15 @@ main (int argc, char **argv)
 	 */
 	server = soup_test_server_new (TRUE);
 	soup_server_add_handler (server, NULL, server_handler, NULL, NULL);
-	uri = soup_test_server_get_uri (server, "http", NULL);
-	port = uri->port;
-	soup_uri_set_scheme (uri, SOUP_URI_SCHEME_HTTPS);
-	soup_uri_set_port (uri, port);
+	uri = g_strdup_printf ("https://127.0.0.1:%u/",
+			       soup_server_get_port (server));
 
-	g_test_add_func ("/no-ssl/session-properties", do_session_property_tests);
-	g_test_add_data_func ("/no-ssl/request-error", uri, do_ssl_tests);
+	do_session_property_tests ();
+	do_ssl_tests (uri);
 
-	ret = g_test_run ();
-
-	soup_uri_free (uri);
+	g_free (uri);
 	soup_test_server_quit_unref (server);
 
 	test_cleanup ();
-	return ret;
+	return errors != 0;
 }
