@@ -21,16 +21,6 @@ static struct {
   { ".example", NULL },
   { ".example.com", NULL },
   { ".example.example", NULL },
-  /* Unlisted TLD.*/
-  { "example", NULL },
-  { "example.example", NULL },
-  { "b.example.example", NULL },
-  { "a.b.example.example", NULL },
-  /* Listed, but non-Internet, TLD. */
-  { "local", NULL },
-  { "example.local", NULL },
-  { "b.example.local", NULL },
-  { "a.b.example.local", NULL },
   /* TLD with only 1 rule. */
   { "biz", NULL },
   { "domain.biz", "domain.biz" },
@@ -67,12 +57,12 @@ static struct {
   { "city.kyoto.jp", "city.kyoto.jp" },
   { "www.city.kyoto.jp", "city.kyoto.jp" },
   /* TLD with a wildcard rule and exceptions. */
-  { "om", NULL },
-  { "test.om", NULL },
-  { "b.test.om", "b.test.om" },
-  { "a.b.test.om", "b.test.om" },
-  { "songfest.om", "songfest.om" },
-  { "www.songfest.om", "songfest.om" },
+  { "ck", NULL },
+  { "test.ck", NULL },
+  { "b.test.ck", "b.test.ck" },
+  { "a.b.test.ck", "b.test.ck" },
+  { "www.ck", "www.ck" },
+  { "www.www.ck", "www.ck" },
   /* US K12. */
   { "us", NULL },
   { "test.us", "test.us" },
@@ -83,11 +73,32 @@ static struct {
   { "k12.ak.us", NULL },
   { "test.k12.ak.us", "test.k12.ak.us" },
   { "www.test.k12.ak.us", "test.k12.ak.us" },
+  /* IDN labels. */
+  { "食狮.com.cn", "食狮.com.cn" },
+  { "食狮.公司.cn", "食狮.公司.cn" },
+  { "www.食狮.公司.cn", "食狮.公司.cn" },
+  { "shishi.公司.cn", "shishi.公司.cn" },
+  { "公司.cn", NULL },
+  { "食狮.中国", "食狮.中国" },
+  { "www.食狮.中国", "食狮.中国" },
+  { "shishi.中国", "shishi.中国" },
+  { "中国", NULL },
   /* This is not in http://publicsuffix.org/list/test.txt but we want to check it anyway. */
   { "co.uk", NULL },
-  /* The original list does not include non-ASCII tests. Let's add a couple. */
-  { "公司.cn", NULL },
-  { "a.b.åfjord.no", "b.åfjord.no" }
+},
+/* Non Internet TLDs have NULL as expected result
+ */
+non_inet_tld_tests[] = {
+  /* Unlisted TLD.*/
+  { "example", NULL },
+  { "example.example", NULL },
+  { "b.example.example", NULL },
+  { "a.b.example.example", NULL },
+  /* Listed, but non-Internet, TLD. */
+  { "local", NULL },
+  { "example.local", NULL },
+  { "b.example.local", NULL },
+  { "a.b.example.local", NULL }
 };
 
 int
@@ -99,29 +110,46 @@ main (int argc, char **argv)
 
 	errors = 0;
 	for (i = 0; i < G_N_ELEMENTS (tld_tests); ++i) {
-               gboolean is_public = soup_tld_domain_is_public_suffix (tld_tests[i].hostname);
-               const char *base_domain = soup_tld_get_base_domain (tld_tests[i].hostname, NULL);
+		GError *error = NULL;
+		gboolean is_public = soup_tld_domain_is_public_suffix (tld_tests[i].hostname);
+		const char *base_domain = soup_tld_get_base_domain (tld_tests[i].hostname, &error);
 
-	       debug_printf (1, "Testing %s: ", tld_tests[i].hostname);
-               if (tld_tests[i].result) {
-                       /* Public domains have NULL expected results. */
-                       if (is_public || g_strcmp0 (tld_tests[i].result, base_domain)) {
-                               debug_printf (1, "ERROR: %s got %s (%s expected)\n",
-                                             tld_tests[i].hostname, base_domain, tld_tests[i].result);
-                               ++errors;
-                       } else
-			       debug_printf (1, "OK\n");
-               } else {
-                       /* If there is no expected result then either the domain is public or
-                        * the hostname invalid (for example starts with a leading dot).
-                        */
-                       if (!is_public && base_domain) {
-                               debug_printf (1, "ERROR: public domain %s got %s (none expected)\n",
-                                             tld_tests[i].hostname, base_domain);
-                               ++errors;
-                       } else
-			       debug_printf (1, "OK\n");
-	       }
+		debug_printf (1, "Testing %s: ", tld_tests[i].hostname);
+
+		if (is_public && tld_tests[i].result) {
+			debug_printf (1, "ERROR: domain is public but base_domain is not NULL (%s)\n",
+				      base_domain);
+			++errors;
+		} else if (g_strcmp0 (tld_tests[i].result, base_domain)) {
+			debug_printf (1, "ERROR: %s expected as base domain but got %s\n",
+				      tld_tests[i].result, base_domain);
+			++errors;
+
+		} else if (!tld_tests[i].result && !is_public &&
+			   !g_error_matches (error, SOUP_TLD_ERROR, SOUP_TLD_ERROR_INVALID_HOSTNAME)) {
+			debug_printf (1, "ERROR: not public domain with NULL expected result\n");
+			++errors;
+		} else
+			debug_printf (1, "OK\n");
+
+		g_clear_error(&error);
+	}
+
+	for (i = 0; i < G_N_ELEMENTS (non_inet_tld_tests); ++i) {
+		gboolean is_public = soup_tld_domain_is_public_suffix (non_inet_tld_tests[i].hostname);
+		const char *base_domain = soup_tld_get_base_domain (non_inet_tld_tests[i].hostname, NULL);
+
+		debug_printf (1, "Testing %s: ", non_inet_tld_tests[i].hostname);
+
+		if (is_public) {
+			debug_printf (1, "ERROR: domain incorrectly clasified as public\n");
+			++errors;
+		} else if (base_domain) {
+			debug_printf (1, "ERROR: non NULL base domain (%s) for local url\n",
+				      base_domain);
+			++errors;
+		} else
+			debug_printf (1, "OK\n");
 	}
 
 	test_cleanup ();
